@@ -37,7 +37,7 @@
     .val_default = { .val_str = "128" },       \
     .multi_names = (char const *[]){ "32",  "48", "56", "64", "80", "96", "112", \
                    "128", "160", "192", "224", "256", "320", "384",\
-                   (char*)0 } \
+                   NULL } \
   },
 
 #define ENCODE_PARAM_MP3                 \
@@ -48,7 +48,7 @@
     .val_default = { .val_str = "128" },       \
     .multi_names = (char const *[]){ "32", "40", "48", "56", "64", "80", "96", \
                    "112", "128", "160", "192", "224", "256", "320",\
-                   (char*)0 } \
+                   NULL } \
   },
 
 #define ENCODE_PARAM_AC3 \
@@ -59,7 +59,7 @@
     .val_default = { .val_str = "128" },       \
     .multi_names = (char const *[]){ "32", "40", "48", "56", "64", "80", "96", "112", "128", \
                    "160", "192", "224", "256", "320", "384", "448", "512", \
-                   "576", "640", (char*)0 } \
+                   "576", "640", NULL } \
   },
 
 #define ENCODE_PARAM_WMA \
@@ -68,7 +68,7 @@
     .long_name = TRS("Bit rate (kbps)"),        \
     .type =      BG_PARAMETER_STRINGLIST, \
     .val_default = { .val_str = "128" },       \
-    .multi_names = (char const *[]){ "24", "48", "64", "96", "128", (char*)0 } \
+    .multi_names = (char const *[]){ "24", "48", "64", "96", "128", NULL } \
   },
     
     
@@ -238,6 +238,94 @@ static const bg_parameter_info_t parameters_msmpeg4v3[] = {
   { /* End of parameters */ }
 };
 
+static const bg_parameter_info_t parameters_libx264[] = {
+  {
+    .name =      "libx264_preset",
+    .long_name = TRS("Preset"),
+    .type =      BG_PARAMETER_STRINGLIST,
+    .val_default = {val_str: "medium"},
+    .multi_names = (char const *[]){"ultrafast",
+                                    "superfast",
+                                    "veryfast",
+                                    "faster",
+                                    "fast",
+                                    "medium",
+                                    "slow",
+                                    "slower",
+                                    "veryslow",
+                                    "placebo",
+                                    (char *)0},
+    .multi_labels = (char const *[]){TRS("Ultrafast"),
+                                     TRS("Superfast"),
+                                     TRS("Veryfast"),
+                                     TRS("Faster"),
+                                     TRS("Fast"),
+                                     TRS("Medium"),
+                                     TRS("Slow"),
+                                     TRS("Slower"),
+                                     TRS("Veryslow"),
+                                     TRS("Placebo"),
+                                     (char *)0 },
+  },
+  {
+    .name =      "libx264_tune",
+    .long_name = TRS("Tune"),
+    .type =      BG_PARAMETER_STRINGLIST,
+    .val_default = {val_str: "$none"},
+    .multi_names = (char const *[]){"$none",
+                                    "film",
+                                    "animation",
+                                    "grain",
+                                    "stillimage",
+                                    "psnr",
+                                    "ssim",
+                                    "fastdecode",
+                                    "zerolatency",
+                                    (char *)0},
+    .multi_labels = (char const *[]){TRS("None"),
+                                     TRS("Film"),
+                                     TRS("Animation"),
+                                     TRS("Grain"),
+                                     TRS("Still image"),
+                                     TRS("PSNR"),
+                                     TRS("SSIM"),
+                                     TRS("Fast decode"),
+                                     TRS("Zero latency"),
+                                     (char *)0},
+  },
+  {
+    .name =      "ff_bit_rate_video",
+    .long_name = TRS("Bit rate (kbps)"),
+    .type =      BG_PARAMETER_INT,
+    .val_min     = { .val_i = 0 },
+    .val_max     = { .val_i = 10000 },
+    .val_default = { .val_i = 0 },
+    .help_string = TRS("If > 0 encode with average bitrate"),
+  },
+  {
+    .name =      "libx264_crf",
+    .long_name = TRS("Quality-based VBR"),
+    .type =      BG_PARAMETER_SLIDER_FLOAT,
+    .val_min     = { .val_f = -1.0 },
+    .val_max     = { .val_f = 51.0 },
+    .val_default = { .val_f = 23.0 },
+    .help_string = TRS("Negative means disable"),
+    .num_digits  = 2,
+  },
+  {
+    .name =      "libx264_qp",
+    .long_name = TRS("Force constant QP"),
+    .type =      BG_PARAMETER_SLIDER_INT,
+    .val_min     = { .val_i = -1 },
+    .val_max     = { .val_i = 69 },
+    .val_default = { .val_i = -1 },
+    .help_string = TRS("Negative means disable, 0 means lossless"),
+  },
+  { /* End */ },
+};
+
+/* Audio */
+
 static const bg_parameter_info_t parameters_ac3[] = {
   ENCODE_PARAM_AC3
   { /* End of parameters */ }
@@ -381,6 +469,13 @@ static const ffmpeg_codec_info_t video_codecs[] =
       .parameters = parameters_msmpeg4v3,
       .pixelformats = (enum PixelFormat[]) { PIX_FMT_YUV420P, PIX_FMT_NB },
     },
+    {
+      .name       = "libx264",
+      .long_name  = TRS("H.264"),
+      .id         = CODEC_ID_H264,
+      .parameters = parameters_libx264,
+      .pixelformats = (enum PixelFormat[]) { PIX_FMT_YUV420P, PIX_FMT_NB },
+    },
 #if 0
     {
       .name       = "wmv2",
@@ -406,7 +501,7 @@ add_codec_info(const ffmpeg_codec_info_t ** info, enum CodecID id, int * num)
 
   info = realloc(info, ((*num)+1) * sizeof(*info));
   
-  info[*num] = (ffmpeg_codec_info_t *)0;
+  info[*num] = NULL;
 
   i = 0;
   while(audio_codecs[i].name)
@@ -492,7 +587,7 @@ bg_ffmpeg_create_audio_parameters(const ffmpeg_format_info_t * format_info)
   {
   int i, j, num_infos = 0;
   bg_parameter_info_t * ret;
-  const ffmpeg_codec_info_t ** infos = (const ffmpeg_codec_info_t**)0;
+  const ffmpeg_codec_info_t ** infos = NULL;
   
   /* Create codec array */
   i = 0;
@@ -514,7 +609,7 @@ bg_ffmpeg_create_audio_parameters(const ffmpeg_format_info_t * format_info)
     }
 
   if(!infos)
-    return (bg_parameter_info_t*)0;
+    return NULL;
   
   /* Create parameters */
   ret = bg_parameter_info_copy_array(audio_parameters);
@@ -529,7 +624,7 @@ bg_ffmpeg_create_video_parameters(const ffmpeg_format_info_t * format_info)
   {
   int i, j, num_infos = 0;
   bg_parameter_info_t * ret;
-  const ffmpeg_codec_info_t ** infos = (const ffmpeg_codec_info_t**)0;
+  const ffmpeg_codec_info_t ** infos = NULL;
 
   /* Create codec array */
   i = 0;
@@ -551,7 +646,7 @@ bg_ffmpeg_create_video_parameters(const ffmpeg_format_info_t * format_info)
     }
 
   if(!infos)
-    return (bg_parameter_info_t*)0;
+    return NULL;
   
   /* Create parameters */
 
@@ -775,9 +870,42 @@ const enum_t mb_decision[] =
     found = 1;                                      \
     }
 
+#if LIBAVCODEC_VERSION_MAJOR >= 54
+#define PARAM_DICT_STRING(n, ffmpeg_key) \
+  if(!strcmp(n, name)) \
+    { \
+    if(val->val_str && (val->val_str[0] != '$')) \
+      av_dict_set(options,                      \
+                  ffmpeg_key, val->val_str, 0); \
+    }
+
+#define PARAM_DICT_FLOAT(n, ffmpeg_key) \
+  if(!strcmp(n, name)) \
+    { \
+    char * str; \
+    str = bg_sprintf("%f", val->val_f); \
+    av_dict_set(options, ffmpeg_key, str, 0); \
+    free(str); \
+    }
+
+#define PARAM_DICT_INT(n, ffmpeg_key) \
+  if(!strcmp(n, name)) \
+    { \
+    char * str; \
+    str = bg_sprintf("%d", val->val_i); \
+    av_dict_set(options, ffmpeg_key, str, 0); \
+    free(str); \
+    }
+#endif
+
 
 void
-bg_ffmpeg_set_codec_parameter(AVCodecContext * ctx, const char * name,
+bg_ffmpeg_set_codec_parameter(AVCodecContext * ctx,
+#if LIBAVCODEC_VERSION_MAJOR >= 54
+                              AVDictionary ** options,
+#endif
+
+                              const char * name,
                               const bg_parameter_value_t * val)
   {
   int found = 0, i;
@@ -819,8 +947,6 @@ bg_ffmpeg_set_codec_parameter(AVCodecContext * ctx, const char * name,
   PARAM_FLOAT("ff_p_masking",p_masking);
   PARAM_FLOAT("ff_dark_masking",dark_masking);
   PARAM_ENUM("ff_prediction_method",prediction_method,prediction_method);
-  PARAM_INT("ff_mb_qmin",mb_qmin);
-  PARAM_INT("ff_mb_qmax",mb_qmax);
   PARAM_ENUM("ff_me_cmp",me_cmp,compare_func);
   PARAM_CMP_CHROMA("ff_me_cmp_chroma",me_cmp);
   PARAM_ENUM("ff_me_sub_cmp",me_sub_cmp,compare_func);
@@ -860,11 +986,11 @@ bg_ffmpeg_set_codec_parameter(AVCodecContext * ctx, const char * name,
   PARAM_FLAG("ff_flag_qpel",CODEC_FLAG_QPEL);
   PARAM_FLAG("ff_flag_gmc",CODEC_FLAG_GMC);
   PARAM_FLAG("ff_flag_mv0",CODEC_FLAG_MV0);
-  PARAM_FLAG("ff_flag_part",CODEC_FLAG_PART);
+  //  PARAM_FLAG("ff_flag_part",CODEC_FLAG_PART);
   PARAM_FLAG("ff_flag_gray",CODEC_FLAG_GRAY);
   PARAM_FLAG("ff_flag_emu_edge",CODEC_FLAG_EMU_EDGE);
   PARAM_FLAG("ff_flag_normalize_aqp",CODEC_FLAG_NORMALIZE_AQP);
-  PARAM_FLAG("ff_flag_alt_scan",CODEC_FLAG_ALT_SCAN);
+  //  PARAM_FLAG("ff_flag_alt_scan",CODEC_FLAG_ALT_SCAN);
 #if LIBAVCODEC_VERSION_INT < ((52<<16)+(0<<8)+0)
   PARAM_FLAG("ff_flag_trellis_quant",CODEC_FLAG_TRELLIS_QUANT);
 #else
@@ -872,16 +998,23 @@ bg_ffmpeg_set_codec_parameter(AVCodecContext * ctx, const char * name,
 #endif
   PARAM_FLAG("ff_flag_bitexact",CODEC_FLAG_BITEXACT);
   PARAM_FLAG("ff_flag_ac_pred",CODEC_FLAG_AC_PRED);
-  PARAM_FLAG("ff_flag_h263p_umv",CODEC_FLAG_H263P_UMV);
+  //  PARAM_FLAG("ff_flag_h263p_umv",CODEC_FLAG_H263P_UMV);
   PARAM_FLAG("ff_flag_cbp_rd",CODEC_FLAG_CBP_RD);
   PARAM_FLAG("ff_flag_qp_rd",CODEC_FLAG_QP_RD);
-  PARAM_FLAG("ff_flag_h263p_aiv",CODEC_FLAG_H263P_AIV);
-  PARAM_FLAG("ff_flag_obmc",CODEC_FLAG_OBMC);
+  //  PARAM_FLAG("ff_flag_h263p_aiv",CODEC_FLAG_H263P_AIV);
+  //  PARAM_FLAG("ffx_flag_obmc",CODEC_FLAG_OBMC);
   PARAM_FLAG("ff_flag_loop_filter",CODEC_FLAG_LOOP_FILTER);
-  PARAM_FLAG("ff_flag_h263p_slice_struct",CODEC_FLAG_H263P_SLICE_STRUCT);
+  //  PARAM_FLAG("ff_flag_h263p_slice_struct",CODEC_FLAG_H263P_SLICE_STRUCT);
   PARAM_FLAG("ff_flag_closed_gop",CODEC_FLAG_CLOSED_GOP);
   PARAM_FLAG2("ff_flag2_fast",CODEC_FLAG2_FAST);
   PARAM_FLAG2("ff_flag2_strict_gop",CODEC_FLAG2_STRICT_GOP);
+
+#if LIBAVCODEC_VERSION_MAJOR >= 54
+  PARAM_DICT_STRING("libx264_preset", "preset");
+  PARAM_DICT_STRING("libx264_tune",   "tune");
+  PARAM_DICT_FLOAT("libx264_crf", "crf");
+  PARAM_DICT_FLOAT("libx264_qp", "qp");
+#endif
   
   }
 
