@@ -41,9 +41,11 @@ typedef struct
   bg_flac_t com; /* Must be first for bg_flac_set_audio_parameter */
   
   char * filename;
+
+  FILE * out;
   
   gavl_audio_format_t format;
-  FLAC__FileEncoder * enc;
+  FLAC__StreamEncoder * enc;
 
   /* Metadata stuff */
   
@@ -60,7 +62,21 @@ typedef struct
   int64_t samples_written;
 
   bg_encoder_callbacks_t * cb;
+
+  FLAC__StreamMetadata_StreamInfo si;
   
+  int64_t data_start;
+  int64_t bytes_written;
+
+  /* Table with *all* frames */
+  FLAC__StreamMetadata_SeekPoint * frame_table;
+  
+  uint32_t frame_table_len;
+  uint32_t frame_table_alloc;
+
+  const gavl_compression_info_t * ci;
+
+  int fixed_blocksize;
   } flac_t;
 
 static void * create_flac()
@@ -115,7 +131,7 @@ static void set_parameter_flac(void * data,
                                const bg_parameter_value_t * v)
   {
   flac_t * flac;
-  flac = (flac_t*)data;
+  flac = data;
   
   if(!name)
     return;
@@ -128,32 +144,24 @@ static void set_parameter_flac(void * data,
     flac->num_seektable_entries = v->val_i;
   }
 
-
-static void create_seektable(flac_t * flac)
-  {
-  flac->seektable =
-    FLAC__metadata_object_new(FLAC__METADATA_TYPE_SEEKTABLE);
-
-  FLAC__metadata_object_seektable_template_append_placeholders(flac->seektable,
-                                                               flac->num_seektable_entries);
-  flac->metadata[flac->num_metadata++] = flac->seektable;
-  }
-
 static int open_flac(void * data, const char * filename,
-                     const bg_metadata_t * m, const bg_chapter_list_t * chapter_list)
+                     const gavl_metadata_t * m,
+                     const bg_chapter_list_t * chapter_list)
   {
   int result = 1;
   flac_t * flac;
   
-  flac = (flac_t*)data;
+  flac = data;
 
   /* Create encoder instance */
-  flac->enc = FLAC__file_encoder_new();
+  flac->enc = FLAC__stream_encoder_new();
   
   flac->filename = bg_filename_ensure_extension(filename, "flac");
 
   if(!bg_encoder_cb_create_output_file(flac->cb, flac->filename))
     return 0;
+
+  flac->out = fopen(flac->filename, "wb");
   
   /* Create vorbis comment */
 
@@ -167,23 +175,28 @@ static int open_flac(void * data, const char * filename,
 
   if(flac->use_seektable)
     {
-    create_seektable(flac);
+    flac->seektable =
+      FLAC__metadata_object_new(FLAC__METADATA_TYPE_SEEKTABLE);
+    
+    FLAC__metadata_object_seektable_template_append_placeholders(flac->seektable,
+                                                                 flac->num_seektable_entries);
+    flac->metadata[flac->num_metadata++] = flac->seektable;
     }
   
   
   
   /* Insert metadata */
     
-  FLAC__file_encoder_set_metadata(flac->enc, flac->metadata, flac->num_metadata);
+  FLAC__stream_encoder_set_metadata(flac->enc, flac->metadata, flac->num_metadata);
   return result;
   }
 
 static int add_audio_stream_flac(void * data,
-                                 const char * language,
+                                 const gavl_metadata_t * m,
                                  const gavl_audio_format_t * format)
   {
   flac_t * flac;
-  flac = (flac_t*)data;
+  flac = data;
 
   /* Copy and adjust format */
 
@@ -193,22 +206,83 @@ static int add_audio_stream_flac(void * data,
   return 0;
   }
 
+static void append_packet(flac_t * f, int samples)
+  {
+  if(f->frame_table_len + 1 > f->frame_table_alloc)
+    {
+    f->frame_table_alloc += 10000;
+    f->frame_table = realloc(f->frame_table,
+                             f->frame_table_alloc * sizeof(*f->frame_table));
+    }
+  if(f->frame_table_len)
+    {
+    f->frame_table[f->frame_table_len].sample_number =
+      f->frame_table[f->frame_table_len-1].sample_number +
+      f->frame_table[f->frame_table_len-1].frame_samples;
+    }
+  else
+    {
+    f->frame_table[f->frame_table_len].sample_number = 0;
+    }
+  f->frame_table[f->frame_table_len].frame_samples = samples;
+  f->frame_table[f->frame_table_len].stream_offset = f->bytes_written - f->data_start;
+  f->frame_table_len++;
+  }
+
+
+static void
+metadata_callback(const FLAC__StreamEncoder *decoder,
+                  const FLAC__StreamMetadata *metadata,
+                  void *data)
+  {
+  flac_t * flac = data;
+  
+  if((metadata->type == FLAC__METADATA_TYPE_STREAMINFO) &&
+     !flac->ci)
+    memcpy(&flac->si, &metadata->data.stream_info, sizeof(flac->si));
+  }
+
+static FLAC__StreamEncoderWriteStatus
+write_callback(const FLAC__StreamEncoder *encoder,
+               const FLAC__byte buffer[],
+               size_t bytes,
+               unsigned samples,
+               unsigned current_frame,
+               void *data)
+  {
+  flac_t * flac;
+  flac = data;
+
+  /* Check if we got a real frame */
+  if(samples > 0)
+    {
+    if(flac->data_start < 0)
+      flac->data_start = flac->bytes_written;
+    
+    append_packet(flac, samples);
+    }
+  
+  if(fwrite(buffer, 1, bytes, flac->out) == bytes)
+    {
+    flac->bytes_written += bytes;
+    return FLAC__STREAM_ENCODER_WRITE_STATUS_OK;
+    }
+  return FLAC__STREAM_ENCODER_WRITE_STATUS_FATAL_ERROR;
+  }
+
 static int start_flac(void * data)
   {
   flac_t * flac;
-  flac = (flac_t*)data;
+  flac = data;
 
-  bg_flac_init_file_encoder(&flac->com, flac->enc);
-
-#if BGAV_FLAC_VERSION_INT <= MAKE_VERSION(1, 1, 2)
-  FLAC__file_encoder_set_filename(flac->enc, flac->filename);
-  /* Initialize encoder */
-  if(FLAC__file_encoder_init(flac->enc) != FLAC__FILE_ENCODER_OK)
-#else
-  if(FLAC__stream_encoder_init_file(flac->enc, flac->filename,
-                                    (FLAC__StreamEncoderProgressCallback)0,
-                                    flac) != FLAC__STREAM_ENCODER_OK)
-#endif
+  bg_flac_init_stream_encoder(&flac->com, flac->enc);
+  
+  if(FLAC__stream_encoder_init_stream(flac->enc,
+                                      write_callback,
+                                      NULL, // Seek
+                                      NULL, // Tell
+                                      metadata_callback, // Metadata
+                                      flac) != FLAC__STREAM_ENCODER_OK)
     {
     if(errno)
       bg_log(BG_LOG_ERROR, LOG_DOMAIN, "Initializing encoder failed: %s",
@@ -217,6 +291,21 @@ static int start_flac(void * data)
       bg_log(BG_LOG_ERROR, LOG_DOMAIN, "Initializing encoder failed");
     return 0;
     }
+
+  /* At this point the flac encoder wrote the header, close it here */
+  if(flac->ci)
+    {
+    FLAC__stream_encoder_finish(flac->enc);
+    FLAC__stream_encoder_delete(flac->enc);
+    flac->enc = NULL;
+    }
+  else
+    {
+    flac->com.samples_per_block =
+      FLAC__stream_encoder_get_blocksize(flac->enc);
+    //    fprintf(stderr, "Got blocksize %d\n", flac->com.samples_per_block);
+    }
+  flac->data_start = -1;
   
   return 1;
   }
@@ -227,13 +316,13 @@ static int write_audio_frame_flac(void * data, gavl_audio_frame_t * frame,
   int ret = 1;
   flac_t * flac;
   
-  flac = (flac_t*)data;
+  flac = data;
 
   bg_flac_prepare_audio_frame(&flac->com, frame);
   
   /* Encode */
 
-  if(!FLAC__file_encoder_process(flac->enc, (const FLAC__int32 **) flac->com.buffer,
+  if(!FLAC__stream_encoder_process(flac->enc, (const FLAC__int32 **) flac->com.buffer,
                                  frame->valid_samples))
     ret = 0;
   flac->samples_written += frame->valid_samples;
@@ -244,11 +333,13 @@ static void get_audio_format_flac(void * data, int stream,
                                  gavl_audio_format_t * ret)
   {
   flac_t * flac;
-  flac = (flac_t*)data;
+  flac = data;
   gavl_audio_format_copy(ret, &flac->format);
   
   }
 
+#if 0
+
 /*
  *  Finalize seektable.
  *  It's a bit ugly but we want to encode flac files without having the slightest
@@ -272,12 +363,8 @@ typedef struct
 
   } seektable_client_data;
 
-#if BGAV_FLAC_VERSION_INT <= MAKE_VERSION(1, 1, 2)
 static FLAC__StreamDecoderWriteStatus
-#else
-static FLAC__FileDecoderWriteStatus
-#endif
-seektable_write_callback(const FLAC__FileDecoder *decoder,
+seektable_write_callback(const FLAC__StreamDecoder *decoder,
                          const FLAC__Frame *frame,
                          const FLAC__int32 * const buffer[],
                          void *client_data)
@@ -286,7 +373,7 @@ seektable_write_callback(const FLAC__FileDecoder *decoder,
 
   seektable_client_data * cd;
 
-  cd = (seektable_client_data*)client_data;
+  cd = client_data;
   
   if(cd->sample_position >= cd->seek_samples[cd->table_position])
     {
@@ -302,22 +389,23 @@ seektable_write_callback(const FLAC__FileDecoder *decoder,
       return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
     }
 
-  FLAC__file_decoder_get_decode_position(decoder,
+  FLAC__stream_decoder_get_decode_position(decoder,
                                          &cd->byte_position);
   cd->sample_position += frame->header.blocksize;
   
   return FLAC__STREAM_DECODER_WRITE_STATUS_CONTINUE;
   }
 
-static void seektable_error_callback(const FLAC__FileDecoder *decoder,
+static void seektable_error_callback(const FLAC__StreamDecoder *decoder,
                                      FLAC__StreamDecoderErrorStatus status, void *client_data)
   {
   }
 
-static void seektable_metadata_callback(const FLAC__FileDecoder *decoder,
+static void seektable_metadata_callback(const FLAC__StreamDecoder *decoder,
                                         const FLAC__StreamMetadata *metadata,
                                         void *client_data)
   {
+  
   }
 
 static void finalize_seektable(flac_t * flac)
@@ -325,7 +413,7 @@ static void finalize_seektable(flac_t * flac)
   int i;
   uint64_t index;
   
-  FLAC__FileDecoder       * decoder;
+  FLAC__StreamDecoder       * decoder;
   FLAC__Metadata_Chain    * chain;
   FLAC__Metadata_Iterator * iter;
 
@@ -373,33 +461,24 @@ static void finalize_seektable(flac_t * flac)
   
   /* Populate the seek table */
     
-  decoder = FLAC__file_decoder_new();
-  FLAC__file_decoder_set_md5_checking(decoder, false);
-  FLAC__file_decoder_set_metadata_ignore_all(decoder);
+  decoder = FLAC__stream_decoder_new();
+  FLAC__stream_decoder_set_md5_checking(decoder, false);
+  FLAC__stream_decoder_set_metadata_ignore_all(decoder);
 
-#if BGAV_FLAC_VERSION_INT <= MAKE_VERSION(1, 1, 2)
-  FLAC__file_decoder_set_filename(decoder, flac->filename);
-  FLAC__file_decoder_set_write_callback(decoder, seektable_write_callback);
-  FLAC__file_decoder_set_metadata_callback(decoder, seektable_metadata_callback);
-  FLAC__file_decoder_set_error_callback(decoder, seektable_error_callback);
-  FLAC__file_decoder_set_client_data(decoder, &cd);
-  FLAC__file_decoder_init(decoder);
-#else
   FLAC__stream_decoder_init_file(decoder,
                                  flac->filename,
                                  seektable_write_callback,
                                  seektable_metadata_callback,
                                  seektable_error_callback,
                                  &cd);
-#endif
-
-  FLAC__file_decoder_process_until_end_of_metadata(decoder);
-  FLAC__file_decoder_get_decode_position(decoder, &cd.start_position);
+  
+  FLAC__stream_decoder_process_until_end_of_metadata(decoder);
+  FLAC__stream_decoder_get_decode_position(decoder, &cd.start_position);
 
   cd.byte_position = cd.start_position;
   
-  FLAC__file_decoder_process_until_end_of_file(decoder);
-  FLAC__file_decoder_delete(decoder);
+  FLAC__stream_decoder_process_until_end_of_stream(decoder);
+  FLAC__stream_decoder_delete(decoder);
 
   /* Let the seektable shrink if necessary */
   if(cd.table_size < flac->num_seektable_entries)
@@ -422,10 +501,97 @@ static void finalize_seektable(flac_t * flac)
 
   }
 
+#else
+
+static void build_seek_table(flac_t * flac, FLAC__StreamMetadata * tab)
+  {
+  int i;
+
+  /* We encoded fewer frames than we have in the seektable: Placeholders will remain there */
+  if(flac->frame_table_len <= flac->num_seektable_entries)
+    {
+    for(i = 0; i < flac->frame_table_len; i++)
+      {
+      FLAC__metadata_object_seektable_set_point(tab,
+                                                i, flac->frame_table[i]);
+      }
+    }
+  /* More common case: We have more frames than we will have in the seek table */
+  else
+    {
+    int index = 0;
+    int64_t next_seek_sample;
+    
+    /* First entry is always copied */
+    FLAC__metadata_object_seektable_set_point(tab,
+                                              0, flac->frame_table[0]);
+    index = 1;
+    next_seek_sample = (flac->samples_written * index) / flac->num_seektable_entries;
+    
+    for(i = 1; i < flac->frame_table_len; i++)
+      {
+      if(flac->frame_table[i].sample_number >= next_seek_sample)
+        {
+        FLAC__metadata_object_seektable_set_point(tab,
+                                                  index, flac->frame_table[i]);
+
+        index++;
+        next_seek_sample = (flac->samples_written * index) / flac->num_seektable_entries;
+
+        if(index >= flac->num_seektable_entries)
+          break;
+        }
+      }
+    }
+  }
+
+static void finalize(flac_t * flac)
+  {
+  FLAC__Metadata_Chain    * chain;
+  FLAC__Metadata_Iterator * iter;
+  FLAC__StreamMetadata    * metadata;
+
+  if(!flac->filename)
+    return;
+  
+  chain = FLAC__metadata_chain_new();
+  FLAC__metadata_chain_read(chain, flac->filename);
+  iter = FLAC__metadata_iterator_new();
+  FLAC__metadata_iterator_init(iter, chain);
+
+  /* Update stream info */
+  while(FLAC__metadata_iterator_get_block_type(iter) != FLAC__METADATA_TYPE_STREAMINFO)
+    FLAC__metadata_iterator_next(iter);
+
+  metadata = FLAC__metadata_iterator_get_block(iter);
+  memcpy(&metadata->data.stream_info, &flac->si, sizeof(flac->si));
+
+  /* Seek table */
+  if(flac->seektable) // Build seek table
+    {
+    while(FLAC__metadata_iterator_get_block_type(iter) != FLAC__METADATA_TYPE_SEEKTABLE)
+      FLAC__metadata_iterator_next(iter);
+    metadata = FLAC__metadata_iterator_get_block(iter);
+    build_seek_table(flac, metadata);
+    }
+  
+  /* Write metadata back to the file */
+
+  FLAC__metadata_chain_write(chain,
+                             true, /* use_padding */
+                             true  /* preserve_file_stats */ );
+
+  /* Clean up stuff */
+
+  FLAC__metadata_iterator_delete(iter);
+  FLAC__metadata_chain_delete(chain);
+  }
+#endif
+
 static int close_flac(void * data, int do_delete)
   {
   flac_t * flac;
-  flac = (flac_t*)data;
+  flac = data;
 
   if(flac->seektable && !do_delete)
     {
@@ -433,24 +599,36 @@ static int close_flac(void * data, int do_delete)
     }
   if(flac->enc)
     {
-    FLAC__file_encoder_finish(flac->enc);
-    FLAC__file_encoder_delete(flac->enc);
+    FLAC__stream_encoder_finish(flac->enc);
+    FLAC__stream_encoder_delete(flac->enc);
     flac->enc = NULL;
     }
-  
+
+  if(flac->out)
+    {
+    fclose(flac->out);
+    flac->out = NULL;
+    }
   if(do_delete && flac->filename)
     remove(flac->filename);
-  else if(flac->seektable)
-    finalize_seektable(flac);
+  else
+    finalize(flac);
   
   free(flac->filename);
-  flac->filename = (char*)0;
+  flac->filename = NULL;
   
   if(flac->seektable)
     {
     FLAC__metadata_object_delete(flac->seektable);
     flac->seektable = NULL;
     }
+
+  if(flac->frame_table)
+    {
+    free(flac->frame_table);
+    flac->frame_table = NULL;
+    }
+  
   bg_flac_free(&flac->com);
   return 1;
   }
@@ -458,7 +636,7 @@ static int close_flac(void * data, int do_delete)
 static void destroy_flac(void * priv)
   {
   flac_t * flac;
-  flac = (flac_t*)priv;
+  flac = priv;
   close_flac(priv, 1);
   free(flac);
   }
@@ -468,10 +646,102 @@ static void set_audio_parameter_flac(void * data, int stream,
                                      const bg_parameter_value_t * val)
   {
   flac_t * flac;
-  flac = (flac_t*)data;
+  flac = data;
   bg_flac_set_parameter(&flac->com, name, val);
   }
 
+/* Compressed packet support */
+
+static int writes_compressed_audio_flac(void * priv,
+                            const gavl_audio_format_t * format,
+                            const gavl_compression_info_t * info)
+  {
+  if((info->id == GAVL_CODEC_ID_FLAC) && (info->global_header_len == 42))
+    return 1;
+  return 0;
+  }
+
+static int
+add_audio_stream_compressed_flac(void * priv,
+                                 const gavl_metadata_t * m,
+                                 const gavl_audio_format_t * format,
+                                 const gavl_compression_info_t * info)
+  {
+  uint16_t i_tmp;
+  uint8_t * ptr;
+  flac_t * flac = priv;
+  flac->ci = info;
+
+  gavl_audio_format_copy(&flac->format, format);
+  flac->com.format = &flac->format;
+  
+  ptr = flac->ci->global_header
+    + 8  // Signature + metadata header
+    + 10 // min/max frame/blocksize
+    + 2; // upper 16 bit of 20 samplerate bits
+
+  // |4|3|5|4|
+  i_tmp = ptr[0];
+  i_tmp <<= 8;
+  i_tmp |= ptr[1];
+  
+  flac->si.sample_rate = format->samplerate;
+  flac->si.channels = format->num_channels;
+  flac->si.bits_per_sample = ((i_tmp >> 4) & 0x1f)+1;
+  
+  memcpy(flac->si.md5sum, flac->ci->global_header + 42 - 16, 16);
+  return 0;
+  }
+
+static int write_audio_packet_flac(void * priv, gavl_packet_t * packet, int stream)
+  {
+  flac_t * flac = priv;
+
+  // fprintf(stderr, "%ld %ld\n", packet->duration, flac->samples_written);
+  
+  if(packet->data_len < 6)
+    {
+    bg_log(BG_LOG_ERROR, LOG_DOMAIN, "Packet data too small: %d",
+           packet->data_len);
+    return 0;
+    }
+  
+  if(!flac->samples_written)
+    {
+    flac->fixed_blocksize = !(packet->data[1] & 0x01); // bit 15
+    flac->si.min_blocksize = packet->duration;
+    flac->si.max_blocksize = packet->duration;
+    }
+  else if(!flac->fixed_blocksize)
+    {
+    if(packet->duration < flac->si.min_blocksize)
+      flac->si.min_blocksize = packet->duration;
+    if(packet->duration > flac->si.max_blocksize)
+      flac->si.max_blocksize = packet->duration;
+    }
+  
+  if(!flac->si.min_framesize || (packet->data_len < flac->si.min_framesize))
+    flac->si.min_framesize = packet->data_len;
+  if(packet->data_len > flac->si.max_framesize)
+    flac->si.max_framesize = packet->data_len;
+
+  if(flac->data_start < 0)
+    flac->data_start = flac->bytes_written;
+  
+  append_packet(flac, packet->duration);
+  flac->samples_written += packet->duration;
+
+  flac->si.total_samples = flac->samples_written;
+
+  if(fwrite(packet->data, 1, packet->data_len, flac->out) == packet->data_len)
+    {
+    flac->bytes_written += packet->data_len;
+    return 1;
+    }
+  else
+    return 0;
+  }
+
 const bg_encoder_plugin_t the_plugin =
   {
     .common =
@@ -498,7 +768,11 @@ const bg_encoder_plugin_t the_plugin =
     
     .get_audio_parameters =    bg_flac_get_parameters,
 
+    .writes_compressed_audio = writes_compressed_audio_flac,
+    
     .add_audio_stream =        add_audio_stream_flac,
+    .add_audio_stream_compressed = add_audio_stream_compressed_flac,
+    
     
     .set_audio_parameter =     set_audio_parameter_flac,
 
@@ -507,6 +781,7 @@ const bg_encoder_plugin_t the_plugin =
     .start =                   start_flac,
     
     .write_audio_frame =   write_audio_frame_flac,
+    .write_audio_packet =   write_audio_packet_flac,
     .close =               close_flac
   };
 
