@@ -102,7 +102,7 @@ static void set_callbacks_mpeg(void * data, bg_encoder_callbacks_t * cb)
   }
 
 static int open_mpeg(void * data, const char * filename,
-                     const bg_metadata_t * metadata,
+                     const gavl_metadata_t * metadata,
                      const bg_chapter_list_t * chapter_list)
   {
   e_mpeg_t * e = data;
@@ -115,17 +115,17 @@ static int open_mpeg(void * data, const char * filename,
   /* To make sure this will work, we check for the execuables of
      mpeg2enc, mplex and mp2enc */
 
-  if(!bg_search_file_exec("mpeg2enc", (char**)0))
+  if(!bg_search_file_exec("mpeg2enc", NULL))
     {
     bg_log(BG_LOG_ERROR, LOG_DOMAIN, "Cannot find mpeg2enc exectuable");
     return 0;
     }
-  if(!bg_search_file_exec("mp2enc", (char**)0))
+  if(!bg_search_file_exec("mp2enc", NULL))
     {
     bg_log(BG_LOG_ERROR, LOG_DOMAIN, "Cannot find mp2enc exectuable");
     return 0;
     }
-  if(!bg_search_file_exec("mplex", (char**)0))
+  if(!bg_search_file_exec("mplex", NULL))
     {
     bg_log(BG_LOG_ERROR, LOG_DOMAIN, "Cannot find mplex exectuable");
     return 0;
@@ -134,7 +134,7 @@ static int open_mpeg(void * data, const char * filename,
   }
 
 static int add_audio_stream_mpeg(void * data,
-                                 const char * language,
+                                 const gavl_metadata_t * m,
                                  const gavl_audio_format_t * format)
   {
   e_mpeg_t * e = data;
@@ -156,7 +156,7 @@ static int add_audio_stream_mpeg(void * data,
   }
 
 static int add_audio_stream_compressed_mpeg(void * data,
-                                            const char * language,
+                                            const gavl_metadata_t * m,
                                             const gavl_audio_format_t * format,
                                             const gavl_compression_info_t * ci)
   {
@@ -179,7 +179,9 @@ static int add_audio_stream_compressed_mpeg(void * data,
   }
 
 
-static int add_video_stream_mpeg(void * data, const gavl_video_format_t* format)
+static int add_video_stream_mpeg(void * data,
+                                 const gavl_metadata_t * m,
+                                 const gavl_video_format_t* format)
   {
   e_mpeg_t * e = data;
 
@@ -197,9 +199,11 @@ static int add_video_stream_mpeg(void * data, const gavl_video_format_t* format)
   return (e->num_video_streams - 1);
   }
 
-static int add_video_stream_compressed_mpeg(void * data,
-                                            const gavl_video_format_t* format,
-                                            const gavl_compression_info_t * ci)
+static int
+add_video_stream_compressed_mpeg(void * data,
+                                 const gavl_metadata_t * m,
+                                 const gavl_video_format_t* format,
+                                 const gavl_compression_info_t * ci)
   {
   e_mpeg_t * e = data;
 
@@ -345,7 +349,7 @@ static char * get_filename(e_mpeg_t * e, const char * extension, int is_audio)
       {
       end = start + strlen(start);
       }
-    template = bg_strndup((char*)0, start, end);
+    template = bg_strndup(NULL, start, end);
     }
   else
     {
@@ -386,29 +390,31 @@ static int start_mpeg(void * data)
   
   for(i = 0; i < e->num_audio_streams; i++)
     {
+    if(e->audio_streams[i].ci)
+      bg_mpa_set_ci(&e->audio_streams[i].mpa, e->audio_streams[i].ci);
+    else
+      bg_mpa_set_format(&e->audio_streams[i].mpa, &e->audio_streams[i].format);
+    
     e->audio_streams[i].filename =
       get_filename(e, bg_mpa_get_extension(&e->audio_streams[i].mpa), 1);
 
     if(!e->audio_streams[i].filename)
       return 0;
 
-    if(e->audio_streams[i].ci)
-      bg_mpa_set_ci(&e->audio_streams[i].mpa, e->audio_streams[i].ci);
-    else
-      bg_mpa_set_format(&e->audio_streams[i].mpa, &e->audio_streams[i].format);
     if(!bg_mpa_start(&e->audio_streams[i].mpa, e->audio_streams[i].filename))
       return 0;
     }
   for(i = 0; i < e->num_video_streams; i++)
     {
+    if(e->video_streams[i].ci)
+      bg_mpv_set_ci(&e->video_streams[i].mpv, e->video_streams[i].ci);
+
     e->video_streams[i].filename =
       get_filename(e, bg_mpv_get_extension(&e->video_streams[i].mpv), 0);
 
     if(!e->video_streams[i].filename)
       return 0;
 
-    if(e->video_streams[i].ci)
-      bg_mpv_set_ci(&e->video_streams[i].mpv, e->video_streams[i].ci);
     
     bg_mpv_open(&e->video_streams[i].mpv, e->video_streams[i].filename);
 
@@ -666,10 +672,10 @@ static const bg_parameter_info_t common_parameters[] =
       .long_name = TRS("Format"),
       .type =      BG_PARAMETER_STRINGLIST,
       .val_default = { .val_str = "mpeg1" },
-      .multi_names =    (char const *[]) { "mpeg1",            "vcd",          "mpeg2",            "svcd",         "dvd_nav",   "dvd", (char*)0 },
+      .multi_names =    (char const *[]) { "mpeg1",            "vcd",          "mpeg2",            "svcd",         "dvd_nav",   "dvd", NULL },
       .multi_labels =   (char const *[]) { TRS("MPEG-1 (generic)"), TRS("MPEG-1 (VCD)"),
                                   TRS("MPEG-2 (generic)"), TRS("MPEG-2 (SVCD)"),
-                                  TRS("DVD (NAV)"), TRS("DVD"), (char*)0 },
+                                  TRS("DVD (NAV)"), TRS("DVD"), NULL },
       .help_string = TRS("Output format. Note that for some output formats (e.g. VCD), you MUST use proper settings for the audio and video streams also, since this isn't done automatically")
     },
     {
